=== modified file 'drizzled/cursor.cc'
--- drizzled/cursor.cc	2010-12-20 16:56:48 +0000
+++ drizzled/cursor.cc	2011-01-06 05:55:24 +0000
@@ -278,16 +278,22 @@
   if (stats.deleted < 10 || primary_key >= MAX_KEY ||
       !(getTable()->index_flags(primary_key) & HA_READ_ORDER))
   {
-    (void) startTableScan(1);
-    while ((error= rnd_next(buf)) == HA_ERR_RECORD_DELETED) ;
-    (void) endTableScan();
+    error= startTableScan(1);
+    if (error == 0)
+    {
+      while ((error= rnd_next(buf)) == HA_ERR_RECORD_DELETED) ;
+      (void) endTableScan();
+    }
   }
   else
   {
     /* Find the first row through the primary key */
-    (void) startIndexScan(primary_key, 0);
-    error=index_first(buf);
-    (void) endIndexScan();
+    error= startIndexScan(primary_key, 0);
+    if (error == 0)
+    {
+      error=index_first(buf);
+      (void) endIndexScan();
+    }
   }
   return error;
 }

=== modified file 'drizzled/cursor.h'
--- drizzled/cursor.h	2010-12-24 07:15:43 +0000
+++ drizzled/cursor.h	2011-01-07 04:13:17 +0000
@@ -232,9 +232,9 @@
   /* ha_ methods: pubilc wrappers for private virtual API */
 
   int ha_open(const TableIdentifier &identifier, int mode, int test_if_locked);
-  int startIndexScan(uint32_t idx, bool sorted);
+  int startIndexScan(uint32_t idx, bool sorted) __attribute__ ((warn_unused_result));
   int endIndexScan();
-  int startTableScan(bool scan);
+  int startTableScan(bool scan) __attribute__ ((warn_unused_result));
   int endTableScan();
   int ha_reset();
 
@@ -248,9 +248,9 @@
     and doDeleteRecord() below.
   */
   int ha_external_lock(Session *session, int lock_type);
-  int insertRecord(unsigned char * buf);
-  int updateRecord(const unsigned char * old_data, unsigned char * new_data);
-  int deleteRecord(const unsigned char * buf);
+  int insertRecord(unsigned char * buf) __attribute__ ((warn_unused_result));
+  int updateRecord(const unsigned char * old_data, unsigned char * new_data) __attribute__ ((warn_unused_result));
+  int deleteRecord(const unsigned char * buf) __attribute__ ((warn_unused_result));
   void ha_release_auto_increment();
 
   /** to be actually called to get 'check()' functionality*/
@@ -350,7 +350,7 @@
                                  const unsigned char * key,
                                  key_part_map keypart_map,
                                  enum ha_rkey_function find_flag);
-  virtual int index_next(unsigned char *)
+  virtual int index_next(unsigned char *) __attribute__ ((warn_unused_result))
    { return  HA_ERR_WRONG_COMMAND; }
   virtual int index_prev(unsigned char *)
    { return  HA_ERR_WRONG_COMMAND; }
@@ -535,7 +535,7 @@
     if rnd_init allocates the cursor, second call should position it
     to the start of the table, no need to deallocate and allocate it again
   */
-  virtual int doStartTableScan(bool scan)= 0;
+  virtual int doStartTableScan(bool scan) __attribute__ ((warn_unused_result)) = 0;
   virtual int doEndTableScan() { return 0; }
   virtual int doInsertRecord(unsigned char *)
   {

=== modified file 'drizzled/filesort.cc'
--- drizzled/filesort.cc	2010-12-26 17:38:28 +0000
+++ drizzled/filesort.cc	2011-01-05 13:33:48 +0000
@@ -564,7 +564,8 @@
   if (! indexfile && ! quick_select)
   {
     next_pos=(unsigned char*) 0;			/* Find records in sequence */
-    file->startTableScan(1);
+    if (file->startTableScan(1))
+      return(HA_POS_ERROR);
     file->extra_opt(HA_EXTRA_CACHE, getSession().variables.read_buff_size);
   }
 
@@ -574,7 +575,8 @@
     if (select->quick->reset())
       return(HA_POS_ERROR);
 
-    read_record_info.init_read_record(&getSession(), select->quick->head, select, 1, 1);
+    if (read_record_info.init_read_record(&getSession(), select->quick->head, select, 1, 1))
+      return(HA_POS_ERROR);
   }
 
   /* Remember original bitmaps */

=== modified file 'drizzled/item/subselect.cc'
--- drizzled/item/subselect.cc	2011-01-06 05:17:09 +0000
+++ drizzled/item/subselect.cc	2011-01-09 08:39:30 +0000
@@ -2316,7 +2316,12 @@
   if (table->cursor->inited)
     table->cursor->endIndexScan();
 
-  table->cursor->startTableScan(1);
+  if ((error= table->cursor->startTableScan(1)))
+  {
+    table->print_error(error, MYF(0));
+    return 1;
+  }
+
   table->cursor->extra_opt(HA_EXTRA_CACHE,
                            current_session->variables.read_buff_size);
   table->null_row= 0;
@@ -2493,7 +2498,16 @@
     return(scan_table());
 
   if (!table->cursor->inited)
-    table->cursor->startIndexScan(tab->ref.key, 0);
+  {
+    error= table->cursor->startIndexScan(tab->ref.key, 0);
+
+    if (error != 0)
+    {
+      error= table->report_error(error);
+      return (error != 0);
+    }
+  }
+
   error= table->cursor->index_read_map(table->record[0],
                                      tab->ref.key_buff,
                                      make_prev_keypart_map(tab->ref.key_parts),
@@ -2606,7 +2620,15 @@
     return(scan_table());
 
   if (!table->cursor->inited)
-    table->cursor->startIndexScan(tab->ref.key, 1);
+  {
+    error= table->cursor->startIndexScan(tab->ref.key, 1);
+
+    if (error != 0)
+    {
+      error= table->report_error(error);
+      return(error != 0);
+    }
+  }
   error= table->cursor->index_read_map(table->record[0],
                                      tab->ref.key_buff,
                                      make_prev_keypart_map(tab->ref.key_parts),

=== modified file 'drizzled/optimizer/quick_index_merge_select.cc'
--- drizzled/optimizer/quick_index_merge_select.cc	2010-12-18 04:43:40 +0000
+++ drizzled/optimizer/quick_index_merge_select.cc	2011-01-05 13:33:48 +0000
@@ -177,7 +177,11 @@
   /* index_merge currently doesn't support "using index" at all */
   cursor->extra(HA_EXTRA_NO_KEYREAD);
   /* start table scan */
-  read_record.init_read_record(session, head, (optimizer::SqlSelect*) 0, 1, 1);
+  if ((result= read_record.init_read_record(session, head, (optimizer::SqlSelect*) 0, 1, 1)))
+  {
+    head->print_error(result, MYF(0));
+    return 0;
+  }
   return result;
 }
 

=== modified file 'drizzled/records.cc'
--- drizzled/records.cc	2010-11-08 20:00:45 +0000
+++ drizzled/records.cc	2011-01-06 06:56:27 +0000
@@ -48,10 +48,10 @@
   read_record= rr_sequential;
 }
 
-void ReadRecord::init_read_record_idx(Session *, 
-                                      Table *table_arg,
-                                      bool print_error_arg, 
-                                      uint32_t idx)
+int ReadRecord::init_read_record_idx(Session *,
+                                     Table *table_arg,
+                                     bool print_error_arg,
+                                     uint32_t idx)
 {
   table_arg->emptyRecord();
   table= table_arg;
@@ -61,19 +61,26 @@
 
   table->status=0;			/* And it's always found */
   if (not table->cursor->inited)
-    table->cursor->startIndexScan(idx, 1);
+  {
+    int error= table->cursor->startIndexScan(idx, 1);
+    if (error != 0)
+      return error;
+  }
   /* read_record will be changed to rr_index in rr_index_first */
   read_record= rr_index_first;
+
+  return 0;
 }
 
 
-void ReadRecord::init_read_record(Session *session_arg, 
-                                  Table *table_arg,
-                                  optimizer::SqlSelect *select_arg,
-                                  int use_record_cache, 
-                                  bool print_error_arg)
+int ReadRecord::init_read_record(Session *session_arg,
+                                 Table *table_arg,
+                                 optimizer::SqlSelect *select_arg,
+                                 int use_record_cache,
+                                 bool print_error_arg)
 {
   internal::IO_CACHE *tempfile;
+  int error= 0;
 
   session= session_arg;
   table= table_arg;
@@ -114,7 +121,11 @@
     io_cache->reinit_io_cache(internal::READ_CACHE,0L,0,0);
     ref_pos=table->cursor->ref;
     if (!table->cursor->inited)
-      table->cursor->startTableScan(0);
+    {
+      error= table->cursor->startTableScan(0);
+      if (error != 0)
+        return error;
+    }
 
     /*
       table->sort.addon_field is checked because if we use addon fields,
@@ -146,7 +157,10 @@
   }
   else if (table->sort.record_pointers)
   {
-    table->cursor->startTableScan(0);
+    error= table->cursor->startTableScan(0);
+    if (error != 0)
+      return error;
+
     cache_pos=table->sort.record_pointers;
     cache_end= cache_pos+ table->sort.found_records * ref_length;
     read_record= (table->sort.addon_field ?  rr_unpack_from_buffer : rr_from_pointers);
@@ -154,7 +168,10 @@
   else
   {
     read_record= rr_sequential;
-    table->cursor->startTableScan(1);
+    error= table->cursor->startTableScan(1);
+    if (error != 0)
+      return error;
+
     /* We can use record cache if we don't update dynamic length tables */
     if (!table->no_cache &&
         (use_record_cache > 0 ||
@@ -165,7 +182,7 @@
     }
   }
 
-  return;
+  return 0;
 } /* init_read_record */
 
 

=== modified file 'drizzled/records.h'
--- drizzled/records.h	2010-12-18 04:43:40 +0000
+++ drizzled/records.h	2011-01-06 06:56:27 +0000
@@ -181,11 +181,11 @@
     This is the most basic access method of a table using rnd_init,
     rnd_next and rnd_end. No indexes are used.
 */
-  void init_read_record(Session *session, 
-                        Table *reg_form,
-                        optimizer::SqlSelect *select,
-                        int use_record_cache, 
-                        bool print_errors);
+  int init_read_record(Session *session,
+                       Table *reg_form,
+                       optimizer::SqlSelect *select,
+                       int use_record_cache,
+                       bool print_errors) __attribute__ ((warn_unused_result));
 
   void end_read_record();
 
@@ -205,10 +205,10 @@
                       occurs (except for end-of-records error)
   @param idx          index to scan
                     */
-  void init_read_record_idx(Session *session, 
-                            Table *table,
-                            bool print_error, 
-                            uint32_t idx);
+  int init_read_record_idx(Session *session,
+                           Table *table,
+                           bool print_error,
+                           uint32_t idx) __attribute__ ((warn_unused_result));
 
   void init_reard_record_sequential();
 

=== modified file 'drizzled/sql_delete.cc'
--- drizzled/sql_delete.cc	2010-12-24 07:15:43 +0000
+++ drizzled/sql_delete.cc	2011-01-06 06:56:27 +0000
@@ -241,11 +241,23 @@
 
   if (usable_index==MAX_KEY)
   {
-    info.init_read_record(session,table,select,1,1);
+    if ((error= info.init_read_record(session,table,select,1,1)))
+    {
+      table->print_error(error, MYF(0));
+      delete select;
+      free_underlaid_joins(session, select_lex);
+      return true;
+    }
   }
   else
   {
-    info.init_read_record_idx(session, table, 1, usable_index);
+    if ((error= info.init_read_record_idx(session, table, 1, usable_index)))
+    {
+      table->print_error(error, MYF(0));
+      delete select;
+      free_underlaid_joins(session, select_lex);
+      return true;
+    }
   }
 
   session->set_proc_info("updating");

=== modified file 'drizzled/sql_select.cc'
--- drizzled/sql_select.cc	2011-01-04 20:16:55 +0000
+++ drizzled/sql_select.cc	2011-01-09 08:39:30 +0000
@@ -3053,7 +3053,15 @@
     table->emptyRecord();
     if (table->group && join->tmp_table_param.sum_func_count &&
         table->getShare()->sizeKeys() && !table->cursor->inited)
-      table->cursor->startIndexScan(0, 0);
+    {
+      int tmp_error;
+      tmp_error= table->cursor->startIndexScan(0, 0);
+      if (tmp_error != 0)
+      {
+        table->print_error(tmp_error, MYF(0));
+        return -1;
+      }
+    }
   }
   /* Set up select_end */
   Next_select_func end_select= setup_end_select_func(join);
@@ -3531,7 +3539,11 @@
 
   if (!table->cursor->inited)
   {
-    table->cursor->startIndexScan(tab->ref.key, tab->sorted);
+    error= table->cursor->startIndexScan(tab->ref.key, tab->sorted);
+    if (error != 0)
+    {
+      table->print_error(error, MYF(0));
+    }
   }
 
   /* TODO: Why don't we do "Late NULLs Filtering" here? */
@@ -3579,7 +3591,11 @@
 
   /* Initialize the index first */
   if (!table->cursor->inited)
-    table->cursor->startIndexScan(tab->ref.key, tab->sorted);
+  {
+    error= table->cursor->startIndexScan(tab->ref.key, tab->sorted);
+    if (error != 0)
+      return table->report_error(error);
+  }
 
   /* Perform "Late NULLs Filtering" (see internals manual for explanations) */
   for (uint32_t i= 0 ; i < tab->ref.key_parts ; i++)
@@ -3613,7 +3629,11 @@
   Table *table= tab->table;
 
   if (!table->cursor->inited)
-    table->cursor->startIndexScan(tab->ref.key, tab->sorted);
+  {
+    error= table->cursor->startIndexScan(tab->ref.key, tab->sorted);
+    if (error != 0)
+      return table->report_error(error);
+  }
   if (cp_buffer_from_ref(tab->join->session, &tab->ref))
     return -1;
   if ((error=table->cursor->index_read_last_map(table->getInsertRecord(),
@@ -3728,7 +3748,8 @@
   if (tab->select && tab->select->quick && tab->select->quick->reset())
     return 1;
 
-  tab->read_record.init_read_record(tab->join->session, tab->table, tab->select, 1, true);
+  if (tab->read_record.init_read_record(tab->join->session, tab->table, tab->select, 1, true))
+    return 1;
 
   return (*tab->read_record.read_record)(&tab->read_record);
 }
@@ -3761,7 +3782,14 @@
   }
 
   if (!table->cursor->inited)
-    table->cursor->startIndexScan(tab->index, tab->sorted);
+  {
+    error= table->cursor->startIndexScan(tab->index, tab->sorted);
+    if (error != 0)
+    {
+      table->report_error(error);
+      return -1;
+    }
+  }
   if ((error=tab->table->cursor->index_first(tab->table->getInsertRecord())))
   {
     if (error != HA_ERR_KEY_NOT_FOUND && error != HA_ERR_END_OF_FILE)
@@ -3820,7 +3848,11 @@
   tab->read_record.index=tab->index;
   tab->read_record.record=table->getInsertRecord();
   if (!table->cursor->inited)
-    table->cursor->startIndexScan(tab->index, 1);
+  {
+    error= table->cursor->startIndexScan(tab->index, 1);
+    if (error != 0)
+      return table->report_error(error);
+  }
   if ((error= tab->table->cursor->index_last(tab->table->getInsertRecord())))
     return table->report_error(error);
 
@@ -5099,7 +5131,9 @@
   org_record=(char*) (record=table->getInsertRecord())+offset;
   new_record=(char*) table->getUpdateRecord()+offset;
 
-  cursor->startTableScan(1);
+  if ((error= cursor->startTableScan(1)))
+    goto err;
+
   error=cursor->rnd_next(record);
   for (;;)
   {
@@ -5216,7 +5250,9 @@
     return(1);
   }
 
-  cursor->startTableScan(1);
+  if ((error= cursor->startTableScan(1)))
+    goto err;
+
   key_pos= &key_buffer[0];
   for (;;)
   {

=== modified file 'drizzled/sql_update.cc'
--- drizzled/sql_update.cc	2011-01-04 11:36:01 +0000
+++ drizzled/sql_update.cc	2011-01-09 08:39:30 +0000
@@ -136,7 +136,7 @@
   bool		using_limit= limit != HA_POS_ERROR;
   bool		used_key_is_modified;
   bool		transactional_table;
-  int		error;
+  int		error= 0;
   uint		used_index= MAX_KEY, dup_key_found;
   bool          need_sort= true;
   ha_rows	updated, found;
@@ -363,11 +363,13 @@
 
       if (used_index == MAX_KEY || (select && select->quick))
       {
-        info.init_read_record(session, table, select, 0, true);
+        if ((error= info.init_read_record(session, table, select, 0, true)))
+          goto err;
       }
       else
       {
-        info.init_read_record_idx(session, table, 1, used_index);
+        if ((error= info.init_read_record_idx(session, table, 1, used_index)))
+          goto err;
       }
 
       session->set_proc_info("Searching rows for update");
@@ -433,7 +435,10 @@
   if (select && select->quick && select->quick->reset())
     goto err;
   table->cursor->try_semi_consistent_read(1);
-  info.init_read_record(session, table, select, 0, true);
+  if ((error= info.init_read_record(session, table, select, 0, true)))
+  {
+    goto err;
+  }
 
   updated= found= 0;
   /*
@@ -580,6 +585,9 @@
   return ((error >= 0 || session->is_error()) ? 1 : 0);
 
 err:
+  if (error != 0)
+    table->print_error(error,MYF(0));
+
   delete select;
   free_underlaid_joins(session, select_lex);
   if (table->key_read)

=== modified file 'drizzled/statement/alter_table.cc'
--- drizzled/statement/alter_table.cc	2011-01-08 10:52:13 +0000
+++ drizzled/statement/alter_table.cc	2011-01-09 08:39:30 +0000
@@ -1529,7 +1529,12 @@
 
   /* Tell handler that we have values for all columns in the to table */
   to->use_all_columns();
-  info.init_read_record(session, from, (optimizer::SqlSelect *) 0, 1, true);
+  error= info.init_read_record(session, from, (optimizer::SqlSelect *) 0, 1, true);
+  if (error)
+  {
+    to->print_error(errno, MYF(0));
+    goto err;
+  }
 
   if (ignore)
     to->cursor->extra(HA_EXTRA_IGNORE_DUP_KEY);

=== modified file 'plugin/haildb/haildb_engine.cc'
--- plugin/haildb/haildb_engine.cc	2010-12-30 18:10:25 +0000
+++ plugin/haildb/haildb_engine.cc	2011-01-05 14:00:54 +0000
@@ -2212,14 +2212,13 @@
   err= ib_cursor_first(cursor);
   if (err != DB_SUCCESS && err != DB_END_OF_INDEX)
   {
-    previous_error= ib_err_t_to_drizzle_error(err);
-    err= ib_cursor_reset(cursor);
-    return previous_error;
+    int reset_err= ib_cursor_reset(cursor);
+    assert(reset_err == DB_SUCCESS);
+    return ib_err_t_to_drizzle_error(err);
   }
 
   advance_cursor= false;
 
-  previous_error= 0;
   return(0);
 }
 
@@ -2315,9 +2314,6 @@
   ib_err_t err;
   int ret;
 
-  if (previous_error)
-    return previous_error;
-
   if (advance_cursor)
   {
     err= ib_cursor_next(cursor);
@@ -2343,7 +2339,6 @@
   err= ib_cursor_reset(cursor);
   assert(err == DB_SUCCESS);
   in_table_scan= false;
-  previous_error= 0;
   return ib_err_t_to_drizzle_error(err);
 }
 

=== modified file 'plugin/haildb/haildb_engine.h'
--- plugin/haildb/haildb_engine.h	2010-12-13 06:37:07 +0000
+++ plugin/haildb/haildb_engine.h	2011-01-05 14:00:54 +0000
@@ -122,7 +122,6 @@
   ib_tpl_t tuple;
   bool advance_cursor;
   ib_lck_mode_t ib_lock_mode;
-  int previous_error;
   bool cursor_is_sec_index;
 
   bool write_can_replace;

